// minimal chorus effect by jl 2003
// Version 1.0, 2003-05-22

#include "stdafx.h"
#include <iostream>
#include <fstream>
#include <cmath>
using namespace std;

const double PI = 4.0*atan(1.0);

#pragma pack(1) // tell MSVC not to leave gaps in data structures
struct wavHeader
{
	// We assume that short has 16 bits and long has 32 bits.
	unsigned long riffString; // 'FFIR' (litte endian!) 
	long fileLength; // length - 8
	unsigned long waveString; // 'EVAW'
	unsigned long fmtString; // ' tmf'
	long fmtLength; // 16
	short formatTag; // 1 = PCM
	short channels; // 1, 2, ...
	long sampleRate; // samples per second, e.g. 44100
	long bytesPerSecond; // sampleRate*blockAlign
	short blockAlign; // (channels*bitsPerSample)/8
	short bitsPerSample; // 8, 16, ...
	unsigned long dataString; // 'atad'
	long dataLength;
};
#pragma pack()
// Don't forget: On big endian systems (i.e. Macs) we have to shuffle bytes!

int main(int argc, char* argv[])
{
	ifstream inFile("in.wav", ios::in | ios::binary);
	if(!inFile)
	{
		cerr << "Can't open in.wav" << endl;
		exit(-1);
	}

	wavHeader inHeader;
	inFile.read((char*) &inHeader, sizeof(inHeader));

	// code to add: check if header is filled as expected
	// code to add: check for file errors

	if(inHeader.channels != 2 || inHeader.bitsPerSample != 16)
	{
		cerr << "No standard 16 bit stereo file" << endl;
		exit(-2);
	}

	long numberOfSamples = inHeader.dataLength/4;
	short (*samples)[2] = new short[numberOfSamples][2];
	inFile.read((char*) samples, inHeader.dataLength);

	// code to add: check for file errors

	short (*outSamples)[2] = new short[numberOfSamples][2];

	const double baseDelay = 0.1; // base delay time (seconds)
	const double modFreq = 0.5; // modulation frequency (Hz)
	const double modInt = 0.005; // modulation intensity (seconds)
	const double wet = 0.8; // how much chorus is mixed in

	int i;
	for(i=0; i<numberOfSamples; ++i)
	{
		int delaySamples = (int)(inHeader.sampleRate*
			(baseDelay+modInt*sin(2.0*PI*modFreq*i/(double)inHeader.sampleRate)));

		// clamp the sample number to admissible range
		int j = i-delaySamples;
		if (j < 0)
			j = 0;
		else if (j >= numberOfSamples)
			j = numberOfSamples-1;

		// we'd better do some real resampling here,
		// at least linear interpolation;
		// clipping behaviour and rounding aren't perfect either
		outSamples[i][0] = samples[i][0] + (int)(wet*samples[j][0]);
		outSamples[i][1] = samples[i][1] + (int)(wet*samples[j][1]);
	}

	ofstream outFile("out.wav", ios::out | ios::binary | ios::trunc);
	
	// somewhat ugly reuse of inHeader
	outFile.write((char*) &inHeader, sizeof(inHeader));
	outFile.write((char*) outSamples, inHeader.dataLength);		

	// code to add: check for file errors

	delete[] outSamples;
	delete[] samples;

	return 0;
}